<?php

    class Windows_Net_RPC_Action extends Reactor_Action
    {
        protected $tpl = 'windows_net_rpc.php';
        protected $name = 'Perform an operation on a Windows machine';
        
        public static $registration = array(
            ACTION_NAME => 'Windows operations (using Net RPC)',
            ACTION_CATEGORY => 'Windows',
            ACTION_DESCRIPTION => 'Runs a "net rpc" command against a windows domain using samba. Requires a context variable.',
            ACTION_AUTHOR => 'Nagios Enterprises, LLC',
            ACTION_VERSION => '1.0',
            ACTION_DATE => '06/11/2014', //Last version update
            ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
            ACTION_TYPE => 'core', //core | user. core modules can't be deleted
            ACTION_META => array(), //Misc information  
        );          
        
        protected $fields = array(
            'server' => array(
                'label' => 'server',
                'validate' => 'trim|required|xss_clean'
            ),
            'server_username' => array(
                'label' => 'username',
                'validate' => 'trim|required|xss_clean'
            ),          
            //base64 encode so we don't have plain text Domain pw's floating around in JSON
            'server_password' => array(
                'label' => 'password',
                'validate' => 'trim|required|base64_encode'
            ),
            'command' => array(
                'label' => 'command',
                'validate' => 'trim|required'
            ),
            'extra' => array(
                'label' => 'extra',
                'validate' => 'trim|xss_clean'
            ),
            'servicename' => array(
                'label' => 'servicename',
                'validate' => 'trim|xss_clean'
            ),
            'username' => array(
                'label' => 'username',
                'validate' => 'trim|xss_clean'
            ),
            'password' => array(
                'label' => 'password',
                'validate' => 'trim|xss_clean'
            ),
            'new_username' => array(
                'label' => 'new username',
                'validate' => 'trim|xss_clean'
            ),
            'groupname' => array(
                'label' => 'groupname',
                'validate' => 'trim|xss_clean'
            ),
            'new_groupname' => array(
                'label' => 'new groupname',
                'validate' => 'trim|xss_clean'
            )
        );

        public function get_label()
        {
            if ($params = $this->get_params()) {
            
                $command = $params['command'];
                $server = $params['server'];
                list($cmdtype, $action) = explode("-", $command, 2);
                $astr = "";
                
                switch($cmdtype)
                {
                    case "user":
                        switch ($action) {
                            case "add":
                                $astr = "Add user account " . $params['username'];
                                break;
                            case "delete":
                                $astr = "Delete user account " . $params['username'];
                                break;
                            case "rename":
                                $astr = "Rename user account " . $params['username'] . " to " . $params['new_username'];
                                break;
                            default:
                                $astr = "Unknown command";
                                break;
                        }
                        return $astr." on Windows host '".$server."'";
                        break;

                    case "group":
                        switch ($action) {
                            case "add":
                                $astr = "Add group " . $params['groupname'];
                                break;
                            case "delete":
                                $astr = "Delete group " . $params['groupname'];
                                break;
                            case "rename":
                                $astr = "Rename group " . $params['groupname'] . " to " . $params['new_groupname'];
                                break;
                            case "add-user":
                                $astr = "Add user " . $params['username'] . " to group " . $params['groupname'];
                                break;
                            case "delete-user":
                                $astr = "Delete user " . $params['username'] . " from group " . $params['groupname'];
                                break;
                            default:
                                $astr = "Unkown command";
                                break;
                        }
                        return $astr." on Windows host '".$server."'";
                        break;

                    case "system":
                        if ($action == "shutdown") {
                            $astr = "Shutdown Windows host '".$server."'";
                        } else if ($action == "reboot") {
                            $astr = "Reboot Windows host '".$server."'";
                        } else {
                            $astr = "Unkown command";
                        }
                        return $astr;
                        break;

                    case "service":
                        switch ($action) {
                            case "start":
                                $astr = "Start service " . $params['servicename'];
                                break;
                            case "stop":
                                $astr = "Stop service " . $params['servicename'];
                                break;
                            case "pause":
                                $astr = "Pause service " . $params['servicename'];
                                break;
                            case "resume":
                                $astr = "Resume service " . $params['servicename'];
                                break;
                            case "delete":
                                $astr = "Delete service " . $params['servicename'];
                                break;
                            default:
                                $astr = "Unkown command";
                                break;
                        }
                        return $astr." on Windows host ".$server."";
                        break;

                    default:
                        break;
                }
            }

            return parent::get_label();
        }
       
        public function run($context)
        {

            if (ENVIRONMENT == 'development') {
                logit("Running windows net rpc...");
            }

            $this->set_context($context);
            $params = $this->get_params();
            $password = base64_decode($params['server_password']);
            list($cmdtype, $action) = explode("-", $params['command'], 2);
            
            //buikld net rpc command
            $cmd = '/usr/bin/net rpc';
            $cmd .= ' -S '.$params['server'];
            $cmd .= ' -U '.$params['server_username'].'%'.$password;

            $error = 0;
            switch($cmdtype) {
                case 'user':
                    if ($action == "add") {
                        if ($this->not_useable($params['username']) || $this->not_useable($params['password'])) {
                            $this->log('error', "Command failed: You must have specified a new username and a new password.");
                            $error = 1;
                        }
                        $cmd .= ' user add ' . $params['username'] . ' ' . $params['password']; // ADD USER
                    } else if ($action == "delete") {
                        if ($this->not_useable($params['username'])) {
                            $this->log('error', "Command failed: You must have specified a username.");
                            $error = 1;
                        }
                        $cmd .= ' user delete ' . $params['username']; // DELETE USER
                    } else if ($action == "rename") {
                        if ($this->not_useable($params['username']) || $this->not_useable($params['new_username'])) {
                            $this->log('error', "Command failed: You must have specified a username and a new username.");
                            $error = 1;
                        }
                        $cmd .= ' user rename ' . $params['username'] . ' ' . $params['new_username']; // RENAME USER
                    }
                    break;
                
                case 'group':

                    if ($this->not_useable($params['groupname'])) {
                        $this->log('error', "Command failed: You must have specified a group name.");
                        $error = 1;
                    }

                    if ($action == "add") {
                        $cmd .= ' group add "' . $params['groupname'] . '"'; // ADD GROUP
                    } else if ($action == "delete") {
                        $cmd .= ' group delete "' . $params['groupname'] . '"'; // DELETE GROUP
                    } else if ($action == "rename") {
                        if ($this->not_useable($params['new_groupname'])) {
                            $this->log('error', "Command failed: You must have specified a new group name.");
                            $error = 1;
                        }
                        $cmd .= ' group rename "' . $params['gorupname'] . '" "' . $params['new_groupname']. '"'; // RENAME GROUP
                    } else if ($action == "add-user") {
                        if ($this->not_useable($params['username'])) {
                            $this->log('error', "Command failed: You must have specified a username.");
                            $error = 1;
                        }
                        $cmd .= ' group addmem "' . $params['groupname'] . '" ' . $params['username']; // ADD USER TO GROUP
                    } else if ($action == "delete-user") {
                        if ($this->not_useable($params['username'])) {
                            $this->log('error', "Command failed: You must have specified a username.");
                            $error = 1;
                        }
                        $cmd .= ' group delmem "' . $params['groupname'] . '" ' . $params['username']; // DELETE USER FROM GROUP
                    }
                    break;
                
                case 'service':

                    // If no servicename... quit
                    if ($this->not_useable($params['servicename'])) {
                        $this->log('error', "Command failed: You must have specified a service name.");
                        $error = 1;
                    }

                    if ($action == "start") {
                        $cmd .= ' service start ' . $params['servicename']; // START SERVICE
                    } else if ($action == "stop") {
                        $cmd .= ' service stop ' . $params['servicename']; // STOP SERVICE
                    } else if ($action == "pause") {
                        $cmd .= ' service pause ' . $params['servicename']; // PAUSE SERVICE
                    } else if ($action == "resume") {
                        $cmd .= ' service resume ' . $params['servicename']; // RESUME SERVICE
                    } else if ($action == "delete") {
                        $cmd .= ' service delete ' . $params['servicename']; // DELETE SERVICE
                    }
                    break;
                
                case 'system':
                    if ($action == "shutdown") {
                        $cmd .= ' shutdown'; // SHUTDOWN SERVER
                    } else if ($action == "reboot") {
                        $cmd .= ' shutdown -r'; // REBOOT SERVER
                    }
                    break;
                
                default:
                    // TODO: Add some sort of error?
                    break;
            }
            
            if (!empty($params['extra'])) {
                $cmd .= ' '.$params['extra'];
            }

            if (ENVIRONMENT == 'development') { logit("CMD: $cmd"); }
            
            if (!$error) {
                $desc = array(0 => array("pipe", "r"), 1 => array("pipe", "w"), 2 => array("pipe", "w"));
                $proc = proc_open($cmd, $desc, $pipes);
                if (is_resource($proc)) {
                    $stdout = stream_get_contents($pipes[1]);
                    $stderr = stream_get_contents($pipes[2]);
                    $code = proc_close($proc);
                } else {
                    $code = 1; // Error if we can't open the proc
                }
            } else {
                $code = 1; // Error out
            }

            if (ENVIRONMENT == 'development') { logit("CMD output: ".$stderr); }
            
            if ($code > 0 && !$error) {
                $this->log('error', "Command Failed: ".str_replace($password, '*******', $cmd));
                $this->log('error', "Command Error: " . $stderr);
            }

            $this->set_result($code);
        }

        // Checks to see if a variable can be used for RPC commands
        private function not_useable($str)
        {
            if (empty($str)) {
                return true;
            } else if (strpos($str, '%') !== false) {
                return true;
            }
        
            return false;
        }

    }